﻿/************************************************************************
 * Applikation  : SPC_2014.Themes (Bezeichnung der Applikation / des Projektes)
 * ---------------------------------------------------------------------
 * Datei        : VerwaltungThemen.cs
 * Beschreibung : Klasse zur Verwaltung/Benutzung der Themen.
 *                Umschaltung und Farben.
 * ---------------------------------------------------------------------
 * Copyright (c) 2013-2023 by
 * LIWO - Prüfautomation
 * Lippok & Wolf GmbH
 * Hutt-Keller-Str. 2
 * 73642 Welzheim, Deutschland
 * 
 * Phone  : +49 7182 93660–0
 * Fax    : +49 7182 93660-99
 * e-mail : info@liwo.com
 * WWW    : http://www.liwo.com
 * 
 * All rights reserved.
 * ---------------------------------------------------------------------
 * 20.12.2013 10:01:51 : Erstellt von SSchmitt.
 * tt.mm.jjjj : Geändert von VV. NNNNNN.
 ************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using Infragistics.Windows.Themes;

namespace WPF_xamGrid_DragDrop_Rows.Themes
{
    //=========================================================================
    /// <summary>
    /// Klasse zur Verwaltung/Benutzung der Themen.
    /// Umschaltung und Farben.
    /// </summary>
    public class VerwaltungThemen
    {
        #region Felder, Eigenschaften
        /// <summary>Merken des aktuellen Themas. Damit wird dann das "alte" Thema gelöscht.</summary>
        private string HauptThemaName = "IG";
        #endregion

        //---------------------------------------------------------------------
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public VerwaltungThemen()
        { }

        //---------------------------------------------------------------------
        /// <summary>
        /// Farb-Thema umschalten!
        /// </summary>
        /// <param name="ThemaPfadName">Name und Pfad der ResourceDictionary.</param>
        private void ThemaUmschalten(string ThemaPfadName)
        {
            // Alle ResourceWasher finden und in einer Liste sammeln.
            List<ResourceWasher> dictResourceWasherList = new List<ResourceWasher>();
            foreach (ResourceDictionary dictionary in Application.Current.Resources.MergedDictionaries)
            {
                if (dictionary.GetType() == typeof(ResourceWasher))
                { dictResourceWasherList.Add(dictionary as ResourceWasher); }
            }

            // Den richtigen ResourceWasher herauspicken.
            string ThemaDateiName = string.Format("{0}.xaml", ThemaPfadName);
            ResourceWasher resourceDictionary = dictResourceWasherList.FirstOrDefault(d => d.SourceDictionary.Source.OriginalString == ThemaDateiName);

            // Wenn er gefunden wurde...
            if (resourceDictionary != null)
            {
                // ...dann entfernen, um ihn durch eine neue Instanz zu ersetzen.
                Application.Current.Resources.MergedDictionaries.Remove(resourceDictionary);
                Application.Current.Resources.MergedDictionaries.Add(resourceDictionary);
            }
            else
            {
                ResourceWasher AddResource = new ResourceWasher();
                ResourceDictionary AddSourceDict = new ResourceDictionary();
                AddSourceDict.Source = new Uri(ThemaDateiName, UriKind.Relative);
                AddResource.SourceDictionary = AddSourceDict;
                Application.Current.Resources.MergedDictionaries.Add(AddResource);
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Thema löschen, z.B. nach einem Wechsel wird der vorherige nicht
        /// mehr gebraucht!
        /// </summary>
        /// <param name="ThemaPfadName">Name und Pfad der ResourceDictionary.</param>
        private void AltesThemaEntfernen(string ThemaPfadName)
        {
            // Alle ResourceWasher finden und in einer Liste sammeln.
            List<ResourceWasher> dictResourceWasherList = new List<ResourceWasher>();
            foreach (ResourceDictionary dictionary in Application.Current.Resources.MergedDictionaries)
            {
                if (dictionary.GetType() == typeof(ResourceWasher))
                { dictResourceWasherList.Add(dictionary as ResourceWasher); }
            }

            // Den richtigen ResourceWasher herauspicken.
            string ThemaDateiName = string.Format("{0}.xaml", ThemaPfadName);
            ResourceWasher resourceDictionary = dictResourceWasherList.FirstOrDefault(d => d.SourceDictionary.Source.OriginalString == ThemaDateiName);

            // Wenn er gefunden wurde...
            if (resourceDictionary != null)
            {
                // ...dann entfernen.
                Application.Current.Resources.MergedDictionaries.Remove(resourceDictionary);
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Schaltet alle Resourcen eines Haupt-Themas um.
        /// </summary>
        /// <param name="ThemaName">Bezeichnung des Haupt-Themas (wie z.B. "MetroDark").</param>
        public void HauptThemaUmschalten(string ThemaName)
        {
            // Hauptpfad des Themas erstellen.
            string HauptPfad = "Themes/" + ThemaName + "/" + ThemaName + ".";

            // Dann die einzelnen Themen umschalten.
            ThemaUmschalten(HauptPfad + "DataVisualization");
            ThemaUmschalten(HauptPfad + "MSControls.Core.Implicit");
            ThemaUmschalten(HauptPfad + "WPF");
            ThemaUmschalten(HauptPfad + "xamGrid");

            if (ThemaName != this.HauptThemaName)
            {
                // Hauptpfad des Themas erstellen.
                string VorherigesThema = "Themes/" + this.HauptThemaName + "/" + this.HauptThemaName + ".";

                // Dann die einzelnen Themen löschen.
                AltesThemaEntfernen(VorherigesThema + "DataVisualization");
                AltesThemaEntfernen(VorherigesThema + "MSControls.Core.Implicit");
                AltesThemaEntfernen(VorherigesThema + "WPF");
                AltesThemaEntfernen(VorherigesThema + "xamGrid");
            }

            // Merken des jetzigen neuen Haupt-Themas.
            this.HauptThemaName = ThemaName;
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Benutzt die ResourceWasher-Klasse um die Farben mit einer neuen Basisfarbe zusammen zu mischen.
        /// </summary>
        /// <param name="NeueBasisFarbe">Die Farbe als Grundlage zum Mischen.</param>
        /// <param name="FarbenModus">Was für ein Modus soll verwendet werden?</param>
        public void ResourceWasherFarbeWechseln(Color NeueBasisFarbe, WashMode FarbenModus)
        {
            foreach (ResourceDictionary dictionary in Application.Current.Resources.MergedDictionaries)
            {
                if (dictionary.GetType() == typeof(ResourceWasher))
                {
                    ResourceWasher IgResource = dictionary as ResourceWasher;
                    if (IgResource != null)
                    {
                        IgResource.WashColor = NeueBasisFarbe;
                        IgResource.WashMode = FarbenModus;
                    }
                }
            }
        }
    }
}
